const std = @import("std");
const mem = std.mem;
const stdout = std.io.getStdOut().writer();
const stderr = std.io.getStdErr().writer();

const utils = @import("utils.zig");
const print = utils.print;
const exit = utils.exit;
const pErr = utils.pErr;

const walkers = @import("walker.zig");
const BitWalker = walkers.BitWalker;
const BitWalkerUint = walkers.BitWalkerUint;

const huffmanstuff = @import("HuffmanGraph.zig");
const HuffmanGraph = huffmanstuff.HuffmanGraph;
const GraphBuilder = huffmanstuff.GraphBuilder;
const create_dynamic_graph = huffmanstuff.create_dynamic_graph;

const zipstuff = @import("./zip.zig");
const ZipFile = zipstuff.ZipFile;

fn usage() void {
    pErr("reader <path>", .{});
    exit("", .{}, 1);
}

pub fn main() !void {
    var args = std.process.args();

    var filePath: ?[]const u8 = null;

    // Skip the current path
    _ = args.next();
    while (args.next()) |arg| {
        if (filePath == null) {
            filePath = arg;
        } else {
            pErr("Invalid argument: {s}", .{arg});
            usage();
        }
    }

    if (filePath == null) {
        pErr("File path not provided. Please provide a path", .{});
        usage();
    }

    var gpa = std.heap.GeneralPurposeAllocator(.{}){};
    defer _ = gpa.deinit();

    var allocator = gpa.allocator();

    var book_path = try std.fs.realpathAlloc(allocator, filePath.?);
    defer allocator.free(book_path);

    var file = std.fs.openFileAbsolute(book_path, .{}) catch |err| {
        exit("Please provide a file path! Error: {?}", .{err}, 1);
        return err;
    };
    defer file.close();

    var stat = try file.stat();

    if (stat.kind != .File) {
        exit("Please provide a valid file", .{}, 1);
    }

    var reader = file.reader();

    var zip_file = try ZipFile.init(allocator, reader);
    //defer zip_file.deinit();

    var first_file = try zip_file.readFile();
    defer first_file.deinit();

    if (!std.mem.eql(u8, first_file.file_name, "mimetype")) {
        exit("Invalid file provided", .{}, 1);
    }

    if (!std.mem.startsWith(u8, first_file.compressed_content, "application/epub")) {
        exit("Invalid file provided", .{}, 1);
    }

    while (true) {
        var second_file = try zip_file.readFile();
        defer second_file.deinit();

        print("File_Path: {s}", .{second_file.file_name});
        if (!mem.endsWith(u8, second_file.file_name, ".jpeg") and !mem.endsWith(u8, second_file.file_name, ".jpg")) {
            print("Data:\n{s}", .{second_file.uncompressed_content});
            print("END File_Path: {s}", .{second_file.file_name});
        }
    }
}
